@@ -28,7 +28,8 @@ from util import Duration
 from log import logDebug, logException, logWarning, m18nc
 from message import Message
 from common import Internal, Debug, Options, StrMixin
-from common import unicodeString, nativeString
+from common import unicodeString, unicode
+from tilesource import TileSource
 from rule import Ruleset
 from game import PlayingGame
 from query import Query
@@ -186,13 +187,11 @@ class Client(pb.Referenceable):
 
     def remote_newTables(self, tables):
         """update table list"""
-        newTables = list(ClientTable(self, *x)
-                         for x in tables)  # pylint: disable=star-args
+        newTables = list(ClientTable(self, *x) for x in tables)
         self.tables.extend(newTables)
         if Debug.table:
-            logDebug(
-                u'%s got new tables:%s' %
-                (nativeString(self.name), newTables))
+            _ = u', '.join(unicode(ClientTable(self, *x)) for x in tables)
+            logDebug(u'%s got new tables:%s' % (self.name, _))
 
     @staticmethod
     def remote_serverRulesets(hashes):
@@ -206,7 +205,7 @@ class Client(pb.Referenceable):
 
     def tableChanged(self, table):
         """update table list"""
-        newTable = ClientTable(self, *table)  # pylint: disable=star-args
+        newTable = ClientTable(self, *table)
         oldTable = self._tableById(newTable.tableid)
         if oldTable:
             self.tables.remove(oldTable)
@@ -271,7 +270,7 @@ class Client(pb.Referenceable):
                     disagree('numbers for played hands: Server:%s, Client:%s' % (
                         self.table.endValues[0], self.game.handctr))
                 for player in self.game.players:
-                    if player.balance != self.table.endValues[1][player.wind]:
+                    if player.balance != self.table.endValues[1][player.wind.char]:
                         disagree('balances for wind %s: Server:%s, Client:%s' % (
                             player.wind, self.table.endValues[1][player.wind], player.balance))
         else:
@@ -447,7 +446,7 @@ class Client(pb.Referenceable):
         if not self.thatWasMe(move.player) and not self.game.playOpen:
             move.player.showConcealedTiles(hadTiles)
         move.player.lastTile = calledTile.exposed
-        move.player.lastSource = 'd'
+        move.player.lastSource = TileSource.LivingWallDiscard
         move.exposedMeld = move.player.exposeMeld(
             hadTiles,
             calledTile=calledTileItem or calledTile)
