@@ -4,7 +4,7 @@
 Copyright (C) 2008-2014 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
 partially based on C++ code from:
-    Copyright (C) 2006 Mauricio Piacentini <mauricio@tabuleiro.com>
+ - Copyright (C) 2006 Mauricio Piacentini <mauricio@tabuleiro.com>
 
 Kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -21,14 +21,18 @@ along with this program if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """
 
+from collections import defaultdict
 
 from qt import QString
 from kde import KConfigSkeleton
-from log import logException
-from common import Internal
+from log import logException, logDebug
+from common import Internal, Debug
+
 
 class Parameter(object):
+
     """helper class for defining configuration parameters"""
+
     def __init__(self, group, name, default=None):
         """configuration group, parameter name, default value"""
         self.group = group
@@ -40,8 +44,11 @@ class Parameter(object):
         """returns the value of this item"""
         return self.item.value()
 
+
 class StringParameter(Parameter):
+
     """helper class for defining string parameters"""
+
     def __init__(self, group, name, default=None):
         if default is None:
             default = ''
@@ -50,14 +57,18 @@ class StringParameter(Parameter):
 
     def add(self, skeleton):
         """add tis parameter to the skeleton"""
-        self.item = skeleton.addItemString(self.name, self.value, QString(self.default or ''))
+        self.item = skeleton.addItemString(
+            self.name, self.value, QString(self.default or ''))
 
     def itemValue(self):
         """returns the value of this item"""
         return str(self.item.value())
 
+
 class BoolParameter(Parameter):
+
     """helper class for defining boolean parameters"""
+
     def __init__(self, group, name, default=None):
         if default is None:
             default = False
@@ -68,9 +79,13 @@ class BoolParameter(Parameter):
         """add tis parameter to the skeleton"""
         self.item = skeleton.addItemBool(self.name, self.value, self.default)
 
+
 class IntParameter(Parameter):
+
     """helper class for defining integer parameters"""
-    def __init__(self, group, name, default=None, minValue=None, maxValue=None):
+
+    def __init__(self, group, name, default=None,
+                 minValue=None, maxValue=None):
         """minValue and maxValue are also used by the edit widget"""
         if default is None:
             default = 0
@@ -87,14 +102,20 @@ class IntParameter(Parameter):
         if self.maxValue is not None:
             self.item.setMaxValue(self.maxValue)
 
+
 class SetupPreferences(KConfigSkeleton):
+
     """Holds all Kajongg options. Only instantiate this once"""
     _Parameters = {}
-    def __init__(self): # pylint: disable=super-init-not-called
+
+    def __init__(self):  # pylint: disable=super-init-not-called
         if Internal.Preferences:
             logException('Preferences is not None')
+        self.__watching = defaultdict(list)
         Internal.Preferences = self
         KConfigSkeleton.__init__(self)
+        self.configChanged.connect(self.callTriggers)
+        self.__oldValues = defaultdict(str)
         self.addString('General', 'tilesetName', 'default')
         self.addString('General', 'windTilesetName', 'traditional')
         self.addString('General', 'backgroundName', 'wood_light')
@@ -106,16 +127,45 @@ class SetupPreferences(KConfigSkeleton):
         self.addBool('Display', 'useSounds', True)
         self.addBool('Display', 'uploadVoice', False)
 
+    def callTrigger(self, name):
+        """call registered callback for this attribute change"""
+        newValue = getattr(self, name)
+        if self.__oldValues[name] != newValue:
+            if Debug.preferences:
+                logDebug(u'{}: {} -> {} calling {}'.format(
+                    name,
+                    self.__oldValues[name],
+                    newValue,
+                    ','.join(x.__name__ for x in self.__watching[name])))
+            for method in self.__watching[name]:
+                method(self.__oldValues[name], newValue)
+        self.__oldValues[name] = newValue
+
+    def callTriggers(self):
+        """call registered callbacks for specific attribute changes"""
+        for name in self.__watching:
+            self.callTrigger(name)
+
+    def addWatch(self, name, method):
+        """If name changes, call method.
+        method must accept 2 arguments: old value and new value."""
+        if method not in self.__watching[name]:
+            self.__watching[name].append(method)
+            if Debug.preferences:
+                logDebug('addWatch on {}, hook {}'.format(
+                    name, method.__name__))
+            self.callTrigger(name)  # initial change
+
     def __getattr__(self, name):
         """undefined attributes might be parameters"""
-        if not name in SetupPreferences._Parameters:
+        if name not in SetupPreferences._Parameters:
             return self.__getattribute__(name)
         par = SetupPreferences._Parameters[name]
         return par.itemValue()
 
     def __setattr__(self, name, value):
         """undefined attributes might be parameters"""
-        if not name in SetupPreferences._Parameters:
+        if name not in SetupPreferences._Parameters:
             KConfigSkeleton.__setattr__(self, name, value)
             return
         par = SetupPreferences._Parameters[name]
@@ -150,9 +200,11 @@ class SetupPreferences(KConfigSkeleton):
         """add a string parameter to the skeleton"""
         self.addParameter(BoolParameter(group, name, default))
 
-    def addInteger(self, group, name, default=None, minValue=None, maxValue=None):
+    def addInteger(self, group, name, default=None,
+                   minValue=None, maxValue=None):
         """add a string parameter to the skeleton"""
-        self.addParameter(IntParameter(group, name, default, minValue, maxValue))
+        self.addParameter(IntParameter(
+            group, name, default, minValue, maxValue))
 
     def animationDuration(self):
         """in milliseconds"""
