@@ -25,7 +25,9 @@ from qt import QGraphicsRectItem, QGraphicsSimpleTextItem
 from qt import QPushButton, QMessageBox, QComboBox
 
 
-from common import Internal, isAlive, WINDS, unicode
+from common import Internal, isAlive, unicode, Debug
+from wind import Wind
+from tilesource import TileSource
 from animation import animate
 from log import logError, logDebug, logWarning, m18n
 from query import Query
@@ -68,7 +70,7 @@ class SelectPlayers(SelectRuleset):
         decorateWindow(self, m18n('Select four players'))
         self.names = None
         self.nameWidgets = []
-        for idx, wind in enumerate(WINDS):
+        for idx, wind in enumerate(Wind.all4):
             cbName = QComboBox()
             cbName.manualSelect = False
             # increase width, we want to see the full window title
@@ -112,7 +114,7 @@ class SelectPlayers(SelectRuleset):
         unusedNames = allNames - self.__selectedNames()
         with BlockSignals(self.nameWidgets):
             used = set([unicode(x.currentText())
-                       for x in self.nameWidgets if x.manualSelect])
+                        for x in self.nameWidgets if x.manualSelect])
             for combo in self.nameWidgets:
                 if not combo.manualSelect:
                     if unicode(combo.currentText()) in used:
@@ -160,8 +162,7 @@ class ScoringHandBoard(HandBoard):
     def meldVariants(self, tile, lowerHalf):
         """Kong might have variants"""
         result = []
-        meld = self.uiMeldWithTile(
-            tile).meld  # pylint: disable=maybe-no-member
+        meld = self.uiMeldWithTile(tile).meld  # pylint: disable=no-member
         result.append(meld.concealed if lowerHalf else meld.exposed)
         if len(meld) == 4:
             if lowerHalf:
@@ -398,20 +399,20 @@ class ScoringPlayer(VisiblePlayer, Player):
     def __mjstring(self):
         """compile hand info into a string as needed by the scoring engine"""
         if self.lastTile and self.lastTile.isConcealed:
-            lastSource = 'w'
+            lastSource = TileSource.LivingWall.char
         else:
-            lastSource = 'd'
-        declaration = ''
+            lastSource = TileSource.LivingWallDiscard.char
+        announcements = set()
         rules = [x.rule for x in self.manualRuleBoxes if x.isChecked()]
         for rule in rules:
             options = rule.options
             if 'lastsource' in options:
-                if lastSource != '1':
+                if lastSource != TileSource.East14th.char:
                     # this defines precedences for source of last tile
                     lastSource = options['lastsource']
-            if 'declaration' in options:
-                declaration = options['declaration']
-        return ''.join(['m', lastSource, declaration])
+            if 'announcements' in options:
+                announcements |= set(options['announcements'])
+        return ''.join(['m', lastSource, ''.join(sorted(announcements))])
 
     def __lastString(self):
         """compile hand info into a string as needed by the scoring engine"""
@@ -423,7 +424,7 @@ class ScoringPlayer(VisiblePlayer, Player):
             return ''
         return 'L%s%s' % (self.lastTile, self.lastMeld)
 
-    def computeHand(self, dummyWithTile=None):
+    def computeHand(self):
         """returns a Hand object, using a cache"""
         self.lastTile = Internal.scene.computeLastTile()
         self.lastMeld = Internal.scene.computeLastMeld()
@@ -446,10 +447,16 @@ class ScoringPlayer(VisiblePlayer, Player):
         """add meld to this hand in a scoring game"""
         if meld.isBonus:
             self._bonusTiles.append(meld[0])
+            if Debug.scoring:
+                logDebug('{} gets bonus tile {}'.format(self, meld[0]))
         elif meld.isConcealed and not meld.isKong:
             self._concealedMelds.append(meld)
+            if Debug.scoring:
+                logDebug('{} gets concealed meld {}'.format(self, meld))
         else:
             self._exposedMelds.append(meld)
+            if Debug.scoring:
+                logDebug('{} gets exposed meld {}'.format(self, meld))
         self._hand = None
 
     def removeMeld(self, uiMeld):
@@ -457,13 +464,14 @@ class ScoringPlayer(VisiblePlayer, Player):
         meld = uiMeld.meld
         if meld.isBonus:
             self._bonusTiles.remove(meld[0])
+            if Debug.scoring:
+                logDebug('{} loses bonus tile {}'.format(self, meld[0]))
         else:
-            popped = False
+            popped = None
             for melds in [self._concealedMelds, self._exposedMelds]:
                 for idx, myMeld in enumerate(melds):
                     if myMeld == meld:
-                        melds.pop(idx)
-                        popped = True
+                        popped = melds.pop(idx)
                         break
             if not popped:
                 logDebug(
@@ -471,6 +479,9 @@ class ScoringPlayer(VisiblePlayer, Player):
                     (self.name, self.__class__.__name__, meld), showStack=3)
                 logDebug(u'    concealed: %s' % self._concealedMelds)
                 logDebug(u'      exposed: %s' % self._exposedMelds)
+            else:
+                if Debug.scoring:
+                    logDebug('{} lost meld {}'.format(self, popped))
         self._hand = None
 
 
@@ -585,12 +596,11 @@ class ScoringGame(Game):
               "VALUES(%d,1,%d,?,?,%d,'%s',%d,'%s','%s',%d,%d,%d,%d,%d)" %
               (self.gameid, self.handctr, player.nameid,
                scoretime, int(player == self.winner),
-               WINDS[self.roundsFinished % 4], player.wind, 0,
+               self.roundWind, player.wind, 0,
                amount, player.balance, self.rotated, self.notRotated),
               (player.hand.string, offense.name))
         Internal.mainWindow.updateGUI()
 
-
 def scoreGame():
     """show all games, select an existing game or create a new game"""
     Players.load()
@@ -610,4 +620,4 @@ def scoreGame():
         selectDialog = SelectPlayers()
         if not selectDialog.exec_():
             return
-        return ScoringGame(list(zip(WINDS, selectDialog.names)), selectDialog.cbRuleset.current)
+        return ScoringGame(list(zip(Wind.all4, selectDialog.names)), selectDialog.cbRuleset.current)
