@@ -22,13 +22,16 @@ from qt import QString, Qt, QRectF, QPointF, QSizeF, QSize, pyqtProperty
 from qt import QGraphicsObject, QGraphicsItem, QPixmap, QPainter, QColor
 
 from util import stack
-from log import logException, logDebug
+from log import logException, logDebug, id4
 from guiutil import Painter
 from common import LIGHTSOURCES, ZValues, Internal, Debug, isAlive
+from common import StrMixin
 from tile import Tile
 from meld import Meld
 
-class UITile(QGraphicsObject):
+
+class UITile(QGraphicsObject, StrMixin):
+
     """A tile visible on the screen. Every tile is only allocated once
     and then reshuffled and reused for every game.
     The unit of xoffset is the width of the tile,
@@ -53,24 +56,22 @@ class UITile(QGraphicsObject):
         self.__yoffset = yoffset
         self.__dark = False
         self.level = level
-        self.activeAnimation = dict() # key is the property name
+        self.activeAnimation = dict()  # key is the property name
         self.queuedAnimations = []
 
-    @property
-    def showShadows(self):
-        """do we need to show shadows?"""
-        return self.board.showShadows if self.board else False
-
     def setClippingFlags(self):
         """if we do not show shadows, we need to clip"""
-        showShadows = self.showShadows
-        self.setFlag(QGraphicsItem.ItemClipsChildrenToShape, enabled=not showShadows)
+        showShadows = Internal.Preferences.showShadows
+        self.setFlag(
+            QGraphicsItem.ItemClipsChildrenToShape,
+            enabled=not showShadows)
         self.setFlag(QGraphicsItem.ItemClipsToShape, enabled=not showShadows)
 
     def keyPressEvent(self, event):
         """redirect to the board"""
-        assert self == self.board.focusTile, 'id(self):%s, self:%s, focusTile:%s/%s' % \
-        (id(self), self, id(self.board.focusTile), self.board.focusTile)
+        if self is not self.board.focusTile:
+            logDebug('id4(self):%s, self:%s, focusTile:%s/%s' % \
+                (id4(self), self, id4(self.board.focusTile), self.board.focusTile))
         return self.board.keyPressEvent(event)
 
     def __lightDistance(self):
@@ -98,7 +99,10 @@ class UITile(QGraphicsObject):
 
     def setDrawingOrder(self):
         """set drawing order for this tile"""
-        boardLevel = self.board.level if self.board else ZValues.boardLevelFactor
+        if self.board:
+            boardLevel = self.board.level
+        else:
+            boardLevel = ZValues.boardLevelFactor
         moving = 0
         # show moving tiles above non-moving tiles
         changePos = self.activeAnimation.get('pos')
@@ -117,16 +121,20 @@ class UITile(QGraphicsObject):
                 newY = changePos.unpackEndValue().x()
             if currentY != newY:
                 moving += ZValues.moving
-        self.setZValue(moving + \
-            boardLevel + \
-            (self.level+(2 if self.tile.isKnown else 1))*ZValues.itemLevelFactor + \
-            self.__lightDistance())
+        self.setZValue(moving +
+                       boardLevel +
+                       (self.level + (2 if self.tile.isKnown else 1))
+                       * ZValues.itemLevelFactor +
+                       self.__lightDistance())
 
     def boundingRect(self):
         """define the part of the tile we want to see. Do not return QRect()
         if tileset is not known because that makes QGraphicsscene crash"""
         if self.tileset:
-            self._boundingRect = QRectF(QPointF(), self.tileset.tileSize if self.showShadows else self.tileset.faceSize)
+            self._boundingRect = QRectF(
+                QPointF(),
+                self.tileset.tileSize if Internal.Preferences.showShadows
+                else self.tileset.faceSize)
         return self._boundingRect
 
     def facePos(self):
@@ -140,10 +148,10 @@ class UITile(QGraphicsObject):
 
     def __elementId(self):
         """returns the SVG element id of the tile"""
-        if not self.showShadows:
+        if not Internal.Preferences.showShadows:
             return QString("TILE_2")
         lightSourceIndex = LIGHTSOURCES.index(self.board.rotatedLightSource())
-        return QString("TILE_{}".format(lightSourceIndex%4+1))
+        return QString("TILE_{}".format(lightSourceIndex % 4 + 1))
 
     def paint(self, painter, dummyOption, dummyWidget=None):
         """paint the entire tile.
@@ -151,7 +159,7 @@ class UITile(QGraphicsObject):
         but that actually made it slower."""
         with Painter(painter):
             renderer = self.tileset.renderer()
-            withBorders = self.showShadows
+            withBorders = Internal.Preferences.showShadows
             if not withBorders:
                 painter.scale(*self.tileset.tileFaceRelation())
             renderer.render(painter, self.__elementId(), self.boundingRect())
@@ -160,10 +168,12 @@ class UITile(QGraphicsObject):
             if self.showFace():
                 if withBorders:
                     faceSize = self.tileset.faceSize.toSize()
-                    renderer.render(painter, self.tileset.svgName[str(self.tile.exposed)],
-                            QRectF(self.facePos(), QSizeF(faceSize)))
+                    renderer.render(
+                        painter, self.tileset.svgName[str(self.tile.exposed)],
+                        QRectF(self.facePos(), QSizeF(faceSize)))
                 else:
-                    renderer.render(painter, self.tileset.svgName[str(self.tile.exposed)],
+                    renderer.render(
+                        painter, self.tileset.svgName[str(self.tile.exposed)],
                         self.boundingRect())
         if self.cross:
             self.__paintCross(painter)
@@ -179,7 +189,8 @@ class UITile(QGraphicsObject):
             painter.drawLine(QPointF(width, 0.0), QPointF(0.0, height))
 
     def pixmapFromSvg(self, pmapSize=None, withBorders=None):
-        """returns a pixmap with default size as given in SVG and optional borders/shadows"""
+        """returns a pixmap with default size as given in SVG
+        and optional borders/shadows"""
         if withBorders is None:
             withBorders = Internal.Preferences.showShadows
         if withBorders:
@@ -192,7 +203,9 @@ class UITile(QGraphicsObject):
         result.fill(Qt.transparent)
         painter = QPainter(result)
         if not painter.isActive():
-            logException('painter is not active. Wanted size: %s' % str(pmapSize))
+            logException(
+                'painter is not active. Wanted size: %s' %
+                str(pmapSize))
         try:
             xScale = float(pmapSize.width()) / wantSize.width()
             yScale = float(pmapSize.height()) / wantSize.height()
@@ -208,10 +221,12 @@ class UITile(QGraphicsObject):
         self._drawDarkness(painter)
         if self.showFace():
             faceSize = self.tileset.faceSize.toSize()
-            faceSize = QSize(faceSize.width() * xScale, faceSize.height() * yScale)
+            faceSize = QSize(
+                faceSize.width() * xScale,
+                faceSize.height() * yScale)
             painter.translate(self.facePos())
             renderer.render(painter, self.tileset.svgName[self.tile.exposed],
-                    QRectF(QPointF(), QSizeF(faceSize)))
+                            QRectF(QPointF(), QSizeF(faceSize)))
         return result
 
     def _drawDarkness(self, painter):
@@ -226,7 +241,7 @@ class UITile(QGraphicsObject):
     def sortKey(self, sortDir=Qt.Key_Right):
         """moving order for cursor"""
         dirs = [Qt.Key_Right, Qt.Key_Up, Qt.Key_Left, Qt.Key_Down] * 2
-        sorter = dirs[dirs.index(sortDir) + self.__board.sceneRotation()//90]
+        sorter = dirs[dirs.index(sortDir) + self.__board.sceneRotation() // 90]
         if sorter == Qt.Key_Down:
             return self.xoffset * 100 + self.yoffset
         elif sorter == Qt.Key_Up:
@@ -265,7 +280,7 @@ class UITile(QGraphicsObject):
         return self._tile
 
     @tile.setter
-    def tile(self, value): # pylint: disable=arguments-differ
+    def tile(self, value):  # pylint: disable=arguments-differ
         """set tile name and update display"""
         if value is not self._tile:
             self._tile = value
@@ -335,24 +350,30 @@ class UITile(QGraphicsObject):
 
     def shortcutAnimation(self, animation):
         """directly set the end value of the animation"""
-        setattr(self, animation.pName(), animation.unpackValue(animation.endValue()))
+        setattr(
+            self,
+            animation.pName(),
+            animation.unpackValue(animation.endValue()))
         self.queuedAnimations = []
         self.setDrawingOrder()
 
     def getValue(self, pName):
         """gets a property value by not returning a QVariant"""
-        return {'pos': self.pos, 'rotation': self.rotation, 'scale':self.scale}[pName]
+        return {'pos': self.pos, 'rotation': self.rotation,
+                'scale': self.scale}[pName]
 
     def setActiveAnimation(self, animation):
         """the tile knows which of its properties are currently animated"""
         self.queuedAnimations = []
         propName = animation.pName()
-        assert propName not in self.activeAnimation or not isAlive(self.activeAnimation[propName])
+        assert propName not in self.activeAnimation or not isAlive(
+            self.activeAnimation[propName])
         self.activeAnimation[propName] = animation
         self.setCacheMode(QGraphicsItem.ItemCoordinateCache)
 
     def clearActiveAnimation(self, animation):
-        """an animation for this tile has ended. Finalize tile in its new position"""
+        """an animation for this tile has ended.
+        Finalize tile in its new position"""
         del self.activeAnimation[animation.pName()]
         self.setDrawingOrder()
         if not len(self.activeAnimation):
@@ -367,9 +388,9 @@ class UITile(QGraphicsObject):
     @focusable.setter
     def focusable(self, value):
         """redirect and generate Debug output"""
-        if str(self) in Debug.focusable:
+        if self.tile in Debug.focusable:
             newStr = 'focusable' if value else 'unfocusable'
-            logDebug('%s: %s from %s' % (newStr, self.tile, stack('')[-2]))
+            logDebug(u'%s: %s from %s' % (newStr, self.tile, stack('')[-2]))
         self.setFlag(QGraphicsItem.ItemIsFocusable, value)
 
     @property
@@ -403,32 +424,30 @@ class UITile(QGraphicsObject):
             if self.__board:
                 self.__board.placeTile(self)
 
-    def __str__(self):
+    def __unicode__(self):
         """printable string with tile"""
-        rotation = ' rot%d' % self.rotation if self.rotation else ''
-        scale = ' scale=%.2f' % self.scale if self.scale != 1 else ''
-        level = ' level=%d' % self.level if self.level else ''
+        rotation = u' rot%d' % self.rotation if self.rotation else u''
+        scale = u' scale=%.2f' % self.scale if self.scale != 1 else u''
+        level = u' level=%d' % self.level if self.level else u''
         if self.boundingRect():
             size = self.boundingRect()
-            size = ' %.2dx%.2d' % (size.width(), size.height())
+            size = u' %.2dx%.2d' % (size.width(), size.height())
         else:
-            size = ''
-        return '%s(%s) %d: x/y/z=%.1f(%.1f)/%.1f(%.1f)/%.2f%s%s%s%s' % \
+            size = u''
+        return u'%s(%s) %d: x/y/z=%.1f(%.1f)/%.1f(%.1f)/%.2f%s%s%s%s' % \
             (self.tile,
-            self.board.name if self.board else 'None', id(self) % 10000,
-            self.xoffset, self.x(), self.yoffset,
-            self.y(), self.zValue(), size, rotation, scale, level)
-
-    def __repr__(self):
-        """default representation"""
-        return 'UITile(%s)' % str(self)
+             self.board.name if self.board else u'None', id(self) % 10000,
+             self.xoffset, self.x(), self.yoffset,
+             self.y(), self.zValue(), size, rotation, scale, level)
 
     @property
     def isBonus(self):
         """proxy for tile"""
         return self.tile.isBonus
 
+
 class UIMeld(list):
+
     """represents a visible meld. Can be empty. Many Meld methods will
     raise exceptions if the meld is empty. But we do not care,
     those methods are not supposed to be called on empty melds.
@@ -438,7 +457,10 @@ class UIMeld(list):
 
     def __init__(self, newContent):
         list.__init__(self)
-        if isinstance(newContent, list) and newContent and isinstance(newContent[0], UITile):
+        if (
+                isinstance(newContent, list)
+                and newContent
+                and isinstance(newContent[0], UITile)):
             self.extend(newContent)
         elif isinstance(newContent, UITile):
             self.append(newContent)
